/////////////////////////////////////////////////////////////////////////////////
// Code file for Earth class
////////////////////////////////////////////////////////////////////////////////

#include "dirty.h"
#include "earth.h"
#include "main.h"
#include <allegro.h>
#include <stdlib.h>

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::Earth
////////////////////////////////////////////////////////////////////////////////
Earth::Earth(int _width, int _height)
{
   set_width(_width);
   set_height(_height);
   set_data(create_bitmap(get_width(), get_height()));
   map = new Earth_element[get_width() * get_height()];
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::~Earth
////////////////////////////////////////////////////////////////////////////////
Earth::~Earth(void)
{
   destroy_bitmap(get_data());
   delete [] map;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::set_data
////////////////////////////////////////////////////////////////////////////////
void Earth::set_data(BITMAP *_data)
{
   data = _data;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::set_data
////////////////////////////////////////////////////////////////////////////////
BITMAP *Earth::get_data(void)
{
   return data;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::set_width
////////////////////////////////////////////////////////////////////////////////
void Earth::set_width(int _width)
{
   width = _width;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::set_height
////////////////////////////////////////////////////////////////////////////////
void Earth::set_height(int _height)
{
   height = _height;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::get_width
////////////////////////////////////////////////////////////////////////////////
int Earth::get_width(void)
{
   return width;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::get_height
////////////////////////////////////////////////////////////////////////////////
int Earth::get_height(void)
{
   return height;
}

////////////////////////////////////////////////////////////////////////////////
// Function: paint_element
////////////////////////////////////////////////////////////////////////////////
void paint_element(BITMAP *bmp, int x, int y, int color)
{
   putpixel(bmp, x, y, (8 * color) + (rand() % 8));
};

////////////////////////////////////////////////////////////////////////////////
// Function: paint_line
////////////////////////////////////////////////////////////////////////////////
void paint_line(BITMAP *bmp, int x, int y, int color)
{
   my_do_vline(bmp, x, y, screen_height, color, paint_element);
};

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::draw
////////////////////////////////////////////////////////////////////////////////
void Earth::draw(BITMAP *bitmap)
{
   masked_blit(get_data(), bitmap, 0, 0, 0, 0, get_width(), get_height());
}

float ran(float max)
{
   return ((float)rand() / (float)RAND_MAX) * max;
}

void bisect(float array[], float x_1, float x_1_val, float x_2, float x_2_val, float range, float roughness)
{
   static int current_depth = 0;

   current_depth++;
   if ((x_2 - x_1) < 0.5) {
      current_depth--;
      return; // End of branch
   }

   array[(int)x_1] = x_1_val;
   array[(int)x_2] = x_2_val;

   float x_middle = (x_1 + x_2) / 2;
   float x_middle_val = ((x_1_val + x_2_val) / 2) + (ran(range * roughness) - ((range * roughness) / 2));

   bisect(array, x_1, x_1_val, x_middle, x_middle_val, range, roughness / 2);
   bisect(array, x_middle, x_middle_val, x_2, x_2_val, range, roughness / 2);
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::generate
////////////////////////////////////////////////////////////////////////////////
void Earth::generate(void)
{
   int _x, _y;
   int old_x, old_y;
   int colour = (rand() % 32) + 1;
   float *layer1 = new float[get_width()];
   float *layer2 = new float[get_width()];
   float *layer3 = new float[get_width()];
   float *layer4 = new float[get_width()];

   highest_earth = get_height();
   clear(get_data());

   bisect(layer1, 0, rand() % (get_height() / 2), get_width(), rand() % (get_height() / 2), 1024, 0.25);
   bisect(layer2, 0, (rand() % (10)) + (10 / 2), get_width(), (rand() % (10 / 2)) + ((10) / 2), 128, 0.5);
   bisect(layer3, 0, (rand() % (10)) + (10 / 2), get_width(), (rand() % (10 / 2)) + ((10) / 2), 64, 0.5);
   bisect(layer4, 0, (rand() % (10)) + (10 / 2), get_width(), (rand() % (10 / 2)) + ((10) / 2), 32, 0.5);
   for (_x = 0; _x < get_width(); _x++) {
      if ((get_height() - ((int)layer1[_x] + (int)layer2[_x] + (int)layer3[_x] + (int)layer4[_x])) < highest_earth) { // Set highest earth
         highest_earth = (get_height() - (((int)layer1[_x] > 0 ? (int)layer1[_x] : 0) + (int)layer2[_x] + (int)layer3[_x] + (int)layer4[_x] + 1));
      }
   }
   for (_x = 0; _x < get_width(); _x++) {
      // Layer 1
      _y = get_height();
      my_do_vline(get_data(), _x, _y - (int)layer1[_x], _y, 30, paint_element);
      // Layer 2
      _y = get_height();
      while ((getpixel(get_data(), _x, _y) != 0) && ( _y > 0)) {
         _y--;
      }
      my_do_vline(get_data(), _x, _y - (int)layer2[_x], _y, 24, paint_element);
      // Layer 3
      _y = get_height();
      while ((getpixel(get_data(), _x, _y) != 0) && ( _y > 0)) {
         _y--;
      }
      my_do_vline(get_data(), _x, _y - (int)layer3[_x], _y, 25, paint_element);
      // Layer 4
      _y = get_height();
      while ((getpixel(get_data(), _x, _y) != 0) && ( _y > 0)) {
         _y--;
      }
      my_do_vline(get_data(), _x, _y - (int)layer4[_x], _y, 12, paint_element);
   }
   delete [] layer1;
   delete [] layer2;
   delete [] layer3;
   delete [] layer4;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::update
////////////////////////////////////////////////////////////////////////////////
void Earth::update(void)
{
   fall();
}

////////////////////////////////////////////////////////////////////////////////
// Function: Earth::fall
////////////////////////////////////////////////////////////////////////////////
void Earth::fall(void)
{
   int _x, _y;
   int a;
   int fall;

   falling.current = falling.head;
   while (falling.current != NULL) {
////// TESTING //
      text_mode(-1);
      textprintf(screen, font_system, falling.current->data->x1,  falling.current->data->y1, -1, "%s", "Falling...");
      rect(screen, falling.current->data->x1, falling.current->data->y1, falling.current->data->x2, falling.current->data->y2, 15);
/////////////////
      if ((falling.current->data->y1 > falling.current->data->y2) || (falling.current->data->exist == FALSE)) {
         delete falling.current->data;
         falling.current = falling.pop(falling.current->data); // Automatically goes to next node
      }
      else {
         fall = 0;
         for (_x = falling.current->data->x2; _x >= falling.current->data->x1; _x--) {
            for (_y = falling.current->data->y2; _y >=  falling.current->data->y1; _y--) {
               a = getpixel(get_data(), _x, _y);
               if (a > 0) {
                  if (((_y + 1) == get_height()) || (getpixel(get_data(), _x, _y + 1) > 0)) {
                     // Is supported so leave
                  }
                  else {
                     putpixel(get_data(), _x, _y + 1, a);
                     putpixel(get_data(), _x, _y, 0);
                     fall++;
                  }
               }
            }
         }
         if (fall == 0) {
            falling.current->data->exist = FALSE;
         }
         falling.current->data->y1++;
         falling.current = falling.current->next; // Go to next node
      }
   }
}

void my_do_hline(BITMAP *bmp, int x1, int y, int x2, int color, void (*proc)(BITMAP *bmp, int x, int y, int color))
{
   int pos_x;
   int temp_x;

   if (x1 > x2) {
      temp_x = x1;
      x1 = x2;
      x2 = temp_x;
   }
   for (pos_x = x1; pos_x <= x2; pos_x++) {
      proc(bmp, pos_x, y, color);
   }
}

void my_do_vline(BITMAP *bmp, int x, int y1, int y2, int color, void (*proc)(BITMAP *bmp, int x, int y, int color))
{
   int pos_y;
   int temp_y;

   if (y1 > y2) {
      temp_y = y1;
      y1 = y2;
      y2 = temp_y;
   }
   for (pos_y = y1; pos_y <= y2; pos_y++) {
      proc(bmp, x, pos_y, color);
   }
}

void my_do_line(BITMAP *bmp, int x1, int y1, int x2, int y2, int color, void (*proc)(BITMAP *bmp, int x, int y, int color))
{
   int start_x, start_y;
   int dist_x, dist_y;
   int pos_x = 0, pos_y = 0;
   int change_x = 0, change_y = 0;

   start_x = x1;
   start_y = y1;
   dist_x = abs(x2 - x1);
   dist_y = abs(y2 - y1);
   // x
   if (x1 < x2) {
      change_x = 1;
   }
   else if (x1 > x2) {
      change_x = -1;
   }
   else { // x1 == x2
      my_do_vline(bmp, x1, y1, y2, color, proc);
      return;
   }
   // y
   if (y1 < y2) {
      change_y = 1;
   }
   else if (y1 > y2) {
      change_y = -1;
   }
   else { // y1 == y2
      my_do_hline(bmp, x1, y1, x2, color, proc);
      return;
   }
   // Draw loop
   while ((pos_x < dist_x) || (pos_y < dist_y)) {
      proc(bmp, start_x + (change_x * pos_x), start_y + (change_y * pos_y), color);
      if ((((float)dist_x - (float)pos_x) / ((float)dist_y - (float)pos_y)) >= ((float)dist_x / (float)dist_y)) {
         pos_x++;
      }
      if ((((float)dist_x - (float)pos_x) / ((float)dist_y - (float)pos_y)) <= ((float)dist_x / (float)dist_y)) {
         pos_y++;
      }
   }
}

void my_do_circle(BITMAP *bmp, int x, int y, int radius, int color, void (*proc)(BITMAP *bmp, int x, int y, int color))
{
   int cx = 0;
   int cy = radius;
   int df = 1 - radius; 
   int d_e = 3;
   int d_se = -2 * radius + 5;

   do {
      proc(bmp, x + cx, y + cy, color);
      if (cx) {
         proc(bmp, x - cx, y + cy, color);
      }
      if (cy) {
         proc(bmp, x + cx, y - cy, color);
      }
      if ((cx) && (cy)) {
         proc(bmp, x - cx, y - cy, color);
      }
      if (cx != cy) {
         proc(bmp, x + cy, y + cx, color);
         if (cx) {
            proc(bmp, x + cy, y - cx, color);
         }
         if (cy) {
            proc(bmp, x - cy, y + cx, color);
         }
         if (cx && cy) {
            proc(bmp, x - cy, y - cx, color);
         }
      }
      if (df < 0)  {
         df += d_e;
         d_e += 2;
         d_se += 2;
      }
      else { 
         df += d_se;
         d_e += 2;
         d_se += 4;
         cy--;
      } 
      cx++;
   } while (cx <= cy);
}

void my_do_circlefill(BITMAP *bmp, int x, int y, int radius, int color, void (*proc)(BITMAP *bmp, int x, int y, int color))
{
   int cx = 0;
   int cy = radius;
   int df = 1 - radius; 
   int d_e = 3;
   int d_se = -2 * radius + 5;

   do {
      my_do_hline(bmp, x - cy, y - cx, x + cy, color, proc);
      if (cx) {
         my_do_hline(bmp, x - cy, y + cx, x + cy, color, proc);
      }
      if (df < 0)  {
         df += d_e;
         d_e += 2;
         d_se += 2;
      }
      else { 
         if (cx != cy) {
            my_do_hline(bmp, x - cx, y - cy, x + cx, color, proc);
            if (cy) {
               my_do_hline(bmp, x - cx, y + cy, x + cx, color, proc);
            }
         }
         df += d_se;
         d_e += 2;
         d_se += 4;
         cy--;
      } 
      cx++;
   } while (cx <= cy);
}

int get_distance(int x1, int y1, int x2, int y2)
{
   double dist_x, dist_y;
   dist_x = (double)abs(x1 - x2);
   dist_y = (double)abs(y1 - y2);
   return (int)sqrt((dist_x * dist_x) + (dist_y * dist_y));
}

void my_do_rectfill(BITMAP *bmp, int x1, int y1, int x2, int y2, int color, void (*proc)(BITMAP *bmp, int x, int y, int color))
{
   if (x2 < x1) {
      x1 ^= x2;
      x2 ^= x1;
      x1 ^= x2;
   }
   if (y2 < y1) {
      y1 ^= y2;
      y2 ^= y1;
      y1 ^= y2;
   }
   while (y1 <= y2) {
      my_do_hline(bmp, x1, y1, x2, color, proc);
      y1++;
   };
}

void my_do_rect(BITMAP *bmp, int x1, int y1, int x2, int y2, int color, void (*proc)(BITMAP *bmp, int x, int y, int color))
{
   if (x2 < x1) {
      x1 ^= x2;
      x2 ^= x1;
      x1 ^= x2;
   }
   if (y2 < y1) {
      y1 ^= y2;
      y2 ^= y1;
      y1 ^= y2;
   }
   my_do_hline(bmp, x1, y1, x2, color, proc);
   if (y2 > y1) {
      my_do_hline(bmp, x1, y2, x2, color, proc);
   }
   if (y2 - 1 >= y1 + 1) {
      my_do_vline(bmp, x1, y1 + 1, y2 - 1, color, proc);
      if (x2 > x1) {
	      my_do_vline(bmp, x2, y1 + 1, y2 - 1, color, proc);
      }
   }
}
