////////////////////////////////////////////////////////////////////////////////
// Primary file for scorch clone
////////////////////////////////////////////////////////////////////////////////

#include <time.h>
#include <stdlib.h>
#include <allegro.h>
#include <iostream.h>
#include "main.h"
#include "tank.h"
#include "missile.h"
#include "explosion.h"
#include "earth.h"
#include "input.h"
#include "keyboard.h"
#include "joystick.h"
#include "mouse.h"
#include "object.h"
#include "singlist.h"

#include "gamedata.h"
#include "mus_data.h"
#include "snd_data.h"

//globals
BITMAP *background;
BITMAP *screen_buffer;
int quit;

COLOR_MAP trans_map;
COLOR_MAP light_map;

int highest_earth = 0;

DATAFILE *game_data;
DATAFILE *snd_data;
DATAFILE *mus_data;
FONT *font_system;
FONT *font_title;
RGB *palette_system;

Earth *ground;
Singlist<Object> *objects;

int screen_width;
int screen_height;

int exit_flag = FALSE;

////////////////////////////////////////////////////////////////////////////////
// Function: generate_sky
////////////////////////////////////////////////////////////////////////////////
void generate_sky(BITMAP *destination, int color)
{
   int row_size = destination->h / 16;

   for (int row = 0; row < 16; row++) {
      rectfill(destination, 0, row * row_size, destination->w, (row + 1) * row_size, (color * 16) + row);
   }
}

////////////////////////////////////////////////////////////////////////////////
// Function: display_startup_screen
////////////////////////////////////////////////////////////////////////////////
void display_startup_screen(void)
{
   clear(screen_buffer);
   generate_sky(screen_buffer, rand() % 16);
   text_mode(-1);
   textprintf(screen_buffer, font_title, 0, text_height(font_title) * 0, -1, "%s", "Scarred Earth");
   textprintf(screen_buffer, font_system, 0, text_height(font_title) * 1, -1, "%s", "Based on Scorch Clone by Nils Oster");
   blit(screen_buffer, screen, 0, 0, 0, 0, screen_width, screen_height);
   play_midi((MIDI *)mus_data[MUSIC01].dat, TRUE);
   while (!keypressed() && !mouse_b) {
     // stuff
   }
   play_midi(NULL, FALSE);
   clear_keybuf();
   clear(screen);
}

////////////////////////////////////////////////////////////////////////////////
// Function: play_loop
////////////////////////////////////////////////////////////////////////////////
void play_loop(void)
{
   while (!key[KEY_ESC]) {
      if (midi_pos < 0) {
         play_midi((MIDI *)mus_data[MUSIC01 + rand() % 9].dat, TRUE);
      }
      clear(screen_buffer);
      blit(background, screen_buffer, 0, 0, 0, 0, screen_width, screen_height);
      // Do earth
      ground->update();
      ground->draw(screen_buffer);
      // Do objects
//      Singlist<Object>::Iterator current_object = objects->head;
//      while (current_object != NULL) {
//         if (current_object->data->get_exist() == FALSE) {
//            delete current_object->data;
//            current_object = objects->pop(current_object->data); // Automatically goes to next node
//         }
//         else {
//            current_object->data->update();
//            current_object->data->draw(screen_buffer);
//            current_object = current_object->next; // Go to next node
//         }
//      }
      Singlist<Object>::Iterator current_object = objects->head;
      while (current_object != NULL) {
         current_object->data->update();
         current_object->data->draw(screen_buffer);
         current_object = current_object->next; // Go to next node
      }
      current_object = objects->head;
      while (current_object != NULL) {
         if (current_object->data->get_exist() == FALSE) {
            delete current_object->data;
            current_object = objects->pop(current_object->data); // Automatically goes to next node
         }
         else {
            current_object = current_object->next; // Go to next node
         }
      }
      // Draw screen_buffer to screen
      text_mode(-1);
      textprintf(screen_buffer, font_system, 0, text_height(font_system) * 0, -1, "Screen size: %d, %d", screen_width, screen_height);
      textprintf(screen_buffer, font_system, 0, text_height(font_system) * 1, -1, "Object count: %d", objects->head->data->Object::get_count());
      textprintf(screen_buffer, font_system, 0, text_height(font_system) * 2, -1, "midi_pos: %d", (int)midi_pos);
      blit(screen_buffer, screen, 0, 0, 0, 0, screen_width, screen_height);
   }
}

////////////////////////////////////////////////////////////////////////////////
// Function: main
////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
   int video_card = GFX_AUTODETECT;
   int video_width = 640;
   int video_height = 480;
   int video_depth = 8;
   Keyboard controller1(KEY_Z, KEY_C, KEY_S, KEY_X, KEY_LSHIFT, KEY_LCONTROL);
   Keyboard controller2(KEY_LEFT, KEY_RIGHT, KEY_UP, KEY_DOWN, KEY_ALTGR, KEY_RCONTROL);
   Joystick controller3(0);
   Mouse controller4(MOUSE_ICONS);
//   Mouse controller4(MOUSE_DIRECT);
   Weapon weapon1("boomer.spr");
   Weapon weapon2("boomer.spr");
   Weapon weapon3("launcher.spr");
   Weapon weapon4("launcher.spr");

   // Initialisation
   srand((int)time(NULL));
   // Initialise Allegro
   allegro_init();
   three_finger_flag = TRUE;
   // Install sound
   if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL)) {
      cout << "Error installing sound!" << endl;
   };
   set_volume(255, 255);
   // Install input devices
   if (install_keyboard()) {
      cout << "Error installing keyboard!" << endl;
   };
   if (!install_mouse()) {
      cout << "Mouse not detected!" << endl;
   };
   if (install_joystick(JOY_TYPE_STANDARD)) {
      cout << "Joystick not detected!" << endl;
   };
   // Install timer
   if (install_timer()) {
      cout << "Error installing timer!" << endl;
   };
   // Initialise data
   if ((game_data = load_datafile("GAMEDATA.DAT")) == NULL) {
      allegro_exit();
      cout << "Error loading datafile!" << endl;
      exit(EXIT_FAILURE);
   }
   // Initialise data
   if ((snd_data = load_datafile("SND_DATA.DAT")) == NULL) {
      allegro_exit();
      cout << "Error loading datafile!" << endl;
      exit(EXIT_FAILURE);
   }
   // Initialise data
   if ((mus_data = load_datafile("MUS_DATA.DAT")) == NULL) {
      allegro_exit();
      cout << "Error loading datafile!" << endl;
      exit(EXIT_FAILURE);
   }
   font_system = (FONT *)game_data[FONT_SYSTEM].dat;
   font_title = (FONT *)game_data[FONT_TITLE].dat;
   palette_system = (PALETTE)game_data[PALETTE_SYSTEM].dat;
   // Initialise video
   set_gfx_mode(GFX_AUTODETECT, 320, 200, 0, 0);
   set_palette(palette_system);
   if (!gfx_mode_select_ex(&video_card, &video_width, &video_height, &video_depth)) {
      allegro_exit();
      cout << "Set video mode cancelled!" << endl;
      exit(EXIT_SUCCESS);
   }
   screen_width = video_width;
   screen_height = video_height;
   if (set_gfx_mode(GFX_AUTODETECT, screen_width, screen_height, 0, 0) < 0) {
      allegro_exit();
      cout << "Error setting video mode!" << endl;
      exit(EXIT_FAILURE);
   }
   // colour
   set_color_depth(video_depth);
   set_palette(palette_system);
   // colour tables
   create_trans_table(&trans_map, palette_system, 63, 63, 63, NULL);
   create_light_table(&light_map, palette_system, 0, 0, 0, NULL);
   // bitmaps
   screen_buffer = create_bitmap(screen_width, screen_height);
   clear(screen_buffer);
   background = create_bitmap(screen_width, screen_height);
   clear(background);
   generate_sky(background, rand() % 16);

   // Set up Earth
   ground = new Earth(screen_width, screen_height);
   ground->generate();
   // Set up objects
   objects = new Singlist<Object>;
   Tank *tank1 = new Tank("kobold.spr", (ground->get_width() / 5) * 1, 0, RED, "Jack", &controller1);
   tank1->weapon = &weapon1;
   objects->push(tank1);
   Tank *tank2 = new Tank("vangar.spr", (ground->get_width() / 5) * 2, 0, YELLOW, "Jane", &controller2);
   tank2->weapon = &weapon2;
   objects->push(tank2);
   Tank *tank3 = new Tank("kobold.spr", (ground->get_width() / 5) * 3, 0, GREEN, "Jim", &controller3);
   tank3->weapon = &weapon3;
   objects->push(tank3);
   Tank *tank4 = new Tank("vangar.spr", (ground->get_width() / 5) * 4, 0, BLUE, "Jill", &controller4);
   tank4->weapon = &weapon4;
   objects->push(tank4);

   // Play loop
   display_startup_screen();
   clear_keybuf();
   play_loop();

   // Cleanup
   delete objects;
   destroy_bitmap(screen_buffer);
   destroy_bitmap(background);
   allegro_exit();

   // Exit
   cout << "Fare thee well, Avatar." << endl << endl;
   
   return EXIT_SUCCESS;
}
