////////////////////////////////////////////////////////////////////////////////
// Code file for Missile class
////////////////////////////////////////////////////////////////////////////////

#include "missile.h"
#include "input.h"
#include "main.h"

#include "gamedata.h"

int Missile::count = 0;

////////////////////////////////////////////////////////////////////////////////
// Function: Missile::Missile
////////////////////////////////////////////////////////////////////////////////
Missile::Missile(char *_sprite_file_name, fix _x, fix _y, fix _x_vel, fix _y_vel)
   : Object(), Sprite()
{
   set_type(TYPE_MISSILE);
   x = _x;
   y = _y;
   x_vel = _x_vel;
   y_vel = _y_vel;
   count++;
   load_file(_sprite_file_name);
}

////////////////////////////////////////////////////////////////////////////////
// Function: Missile::~Missile
////////////////////////////////////////////////////////////////////////////////
Missile::~Missile(void)
{
   count--;
   explode();
}

////////////////////////////////////////////////////////////////////////////////
// Function: Missile::draw
////////////////////////////////////////////////////////////////////////////////
void Missile::draw(BITMAP *b)
{
   Sprite::draw(b, (int)x, (int)y);
   Sprite::next_frame();
}

////////////////////////////////////////////////////////////////////////////////
// Function: Missile::fall
////////////////////////////////////////////////////////////////////////////////
void Missile::fall(void)
{
   y_vel += GRAVITY;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Missile::update
////////////////////////////////////////////////////////////////////////////////
void Missile::update(void)
{
   int modifier = 0;
   if (x_vel > 0) {
      modifier = 64;
   }
   else if (x_vel < 0) {
      modifier = -64;
   }
   else if (x_vel == 0) {
      modifier = -192;
   }
   else {
      modifier = 0;
   }
   Object::angle = atan(y_vel / x_vel) + modifier;
   Sprite::angle = Object::angle;
   x += x_vel;
   y += y_vel;
   fall();

   // check for collsion
   Singlist<Object>::Iterator current_object = objects->head;
   while (current_object != NULL) {
      if (collide(current_object->data->get_x(), current_object->data->get_y()) && (current_object->data->get_type() == TYPE_TANK)) {
         exist = FALSE;
      }
      current_object = current_object->next;
   }

   // Check dead
   if (getpixel(ground->get_data(), (int)get_x(), (int)get_y()) > 0 || ((get_x() < 0) || (get_x() > ground->get_width()) || (get_y() < 0) || (get_y() > ground->get_height()))) {
      exist = FALSE;
   }
}

////////////////////////////////////////////////////////////////////////////////
// Function: Missile::explode
////////////////////////////////////////////////////////////////////////////////
void Missile::explode(void)
{
   Explosion *new_explosion = new Explosion(x, y, EXPLOSION_SIZE);
   objects->push(new_explosion);
}

////////////////////////////////////////////////////////////////////////////////
// Function: Missile::collide
////////////////////////////////////////////////////////////////////////////////
int Missile::collide(fix _x, fix _y)
{
   if ((_x > (x - (get_focus(get_frame()).x * scale))) &&
       (_x < (x - (get_focus(get_frame()).x * scale) + (get_w(get_frame()) * scale))) &&
       (_y > (y - (get_focus(get_frame()).y * scale))) &&
       (_y < (y - (get_focus(get_frame()).y * scale) + (get_h(get_frame()) * scale)))) {
      return 1;
   }
   return 0;
}

