////////////////////////////////////////////////////////////////////////////////
// Code file for Sprite classes
////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <allegro.h>
#include "sprite.h"
#include "main.h"

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::Sprite
////////////////////////////////////////////////////////////////////////////////
Sprite::Sprite(void)
{
   sprite_file_name = NULL;
   image_file_name = NULL;
   frame = 0;
   frames = NULL;
   scale = 0.5;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::~Sprite
////////////////////////////////////////////////////////////////////////////////
Sprite::~Sprite(void)
{
   if (sprite_file_name != NULL) {
      delete [] sprite_file_name;
   }
   if (image_file_name != NULL) {
      delete [] image_file_name;
   }
   for (int i = 0; i < get_frame_count(); i++) {
      destroy_bitmap(frames[i].image);
   }
   if (frames != NULL) {
      delete [] frames;
   }
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::reposition
////////////////////////////////////////////////////////////////////////////////
Vertex Sprite::reposition_vertex(Vertex _vertex)
{
   Vertex temp;

   fix rotation_x = (get_w(get_frame()) / 2) * scale;
   fix rotation_y = (get_h(get_frame()) / 2) * scale;
   fix a1 = (_vertex.x * scale) - rotation_x;
   fix o1 = (_vertex.y * scale) - rotation_y;
   fix d1 = (fix)atan(o1 / a1);
//   fix h1 = a1 / (fix)cos(d1);
   fix h1 = o1 / (fix)sin(d1);
   fix d2 = d1 + angle;
   fix a2 = (fix)cos(d2) * h1;
   fix o2 = (fix)sin(d2) * h1;
   temp.x = rotation_x + a2;
   temp.y = rotation_y + o2;

   return temp;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::draw
////////////////////////////////////////////////////////////////////////////////
void Sprite::draw(BITMAP *destination, int _x, int _y)
{
   Vertex focus_temp = reposition_vertex(get_focus(get_frame()));

   rotate_scaled_sprite(destination, get_image(get_frame()), (fix)(_x - focus_temp.x), (fix)(_y - focus_temp.y), angle.v, scale.v);
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::set_sprite_file_name
////////////////////////////////////////////////////////////////////////////////
void Sprite::set_sprite_file_name(char *_file_name)
{
   if (sprite_file_name != NULL) {
      delete [] sprite_file_name;
   }
   sprite_file_name = new char[strlen(_file_name)];
   strcpy(sprite_file_name, _file_name);
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::set_image_file_name
////////////////////////////////////////////////////////////////////////////////
void Sprite::set_image_file_name(char *_file_name)
{
   if (image_file_name != NULL) {
      delete [] image_file_name;
   }
   image_file_name = new char[strlen(_file_name)];
   strcpy(image_file_name, _file_name);
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::get_sprite_file_name
////////////////////////////////////////////////////////////////////////////////
char *Sprite::get_sprite_file_name(void)
{
   return sprite_file_name;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::get_image_file_name
////////////////////////////////////////////////////////////////////////////////
char *Sprite::get_image_file_name(void)
{
   return image_file_name;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::load_file
////////////////////////////////////////////////////////////////////////////////
int Sprite::load_file(char *_file_name)
{
   FILE *sprite_file;

   if (exists(_file_name)) {
      if ((sprite_file = fopen(_file_name, "rb")) != NULL) {
         set_sprite_file_name(_file_name);
         if (!process_file(sprite_file)) {
         }
         else {
            return 1;
         }
      }
      else {
         return 1;
      }
      fclose(sprite_file);
   }
   else { // file does not exist
      return 1;
   }
   return 0;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Sprite::process_file
////////////////////////////////////////////////////////////////////////////////
int Sprite::process_file(FILE *_file)
{
   int x1, y1, x2, y2, _focus_x, _focus_y;
   int _attachment_x, _attachment_y;
   char buf[80];
   BITMAP *image;

   fscanf(_file, "%s", buf);
   set_image_file_name(buf);
   image = load_bitmap(get_image_file_name(), NULL);
   fscanf(_file, "%d", &frame_count);
   if (frames != NULL) {
      delete [] frames;
   }
   frames = new Sprite_frame [frame_count];
   for (int i = 0; i < frame_count; i++) {
      fscanf(_file, "%d, %d, %d, %d, %d, %d, %d, %d", &x1, &y1, &x2, &y2, &_focus_x, &_focus_y, &_attachment_x, &_attachment_y);
      frames[i].image = create_bitmap(x2 - x1, y2 - y1);
      blit(image, frames[i].image, x1, y1, 0, 0, x2 - x1, y2 - y1);
      frames[i].focus.x = _focus_x - x1;
      frames[i].focus.y = _focus_y - y1;
      frames[i].attachment.x = _attachment_x - x1;
      frames[i].attachment.y = _attachment_y - y1;
   }
   destroy_bitmap(image);
   return 0;
}
