////////////////////////////////////////////////////////////////////////////////
// Code file for Tank class
////////////////////////////////////////////////////////////////////////////////

#include "tank.h"
#include "input.h"
#include "main.h"
#include "math.h"

#include "gamedata.h"

int Tank::count = 0;

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::Tank
////////////////////////////////////////////////////////////////////////////////
Tank::Tank(char *_sprite_file_name, fix _x, fix _y, int _color, char *_name, Input *_controller)
   : Object(), Sprite()
{
   set_type(TYPE_TANK);
   x = _x;
   y = _y;
//   weapon->Object::set_angle(0);
   color = _color;
   set_name(_name);
   count++;
   Object::angle = 0;
   controller = _controller;
   load_file(_sprite_file_name);
   health = 100;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::~Tank
////////////////////////////////////////////////////////////////////////////////
Tank::~Tank(void)
{
   count--;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::draw_info
////////////////////////////////////////////////////////////////////////////////
void Tank::draw_info(BITMAP *destination, int _x, int _y)
{
   text_mode(-1);
   // Backing
   drawing_mode(DRAW_MODE_TRANS, NULL, 0, 0);
   color_map = &trans_map;
   rectfill(destination, (int)_x, (int)_y - text_height(font_system) * 4, (int)_x + (text_length(font_system, name) > text_length(font_system, "000.0") ? text_length(font_system, name) : text_length(font_system, "000.0")), (int)_y - text_height(font_system) * 4 + text_height(font_system) * 4, color);
   drawing_mode(DRAW_MODE_SOLID, NULL, 0, 0);
   rect(destination, (int)_x - 1, (int)_y - text_height(font_system) * 4 - 1, (int)_x + (text_length(font_system, name) > text_length(font_system, "000.0") ? text_length(font_system, name) : text_length(font_system, "000.0")) + 1, (int)_y - text_height(font_system) * 4 + text_height(font_system) * 4 + 1, color);
   // Name
   textprintf(destination, font_system, (int)_x, (int)_y - text_height(font_system) * 4, -1, "%s", name);
   // Power
   textprintf(destination, font_system, (int)_x, (int)_y - text_height(font_system) * 3, -1, "%5.1f", (float)weapon->get_launch_power());
   // Angle
   textprintf(destination, font_system, (int)_x, (int)_y - text_height(font_system) * 2, -1, "%5.1f", (float)weapon->Object::get_angle());
   // Health
   textprintf(destination, font_system, (int)_x, (int)_y - text_height(font_system) * 1, -1, "%d", health);
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::draw
////////////////////////////////////////////////////////////////////////////////
void Tank::draw(BITMAP *b)
{
   Vertex focus_temp = reposition_vertex(get_focus(get_frame()));
   Vertex attachment_temp = reposition_vertex(get_attachment(get_frame()));
   // weapon
   weapon->set_x(x + attachment_temp.x - focus_temp.x);
   weapon->set_y(y + attachment_temp.y - focus_temp.y);
   weapon->update();
   weapon->draw(b);
   putpixel(b, weapon->get_x(), weapon->get_y(), 0);
   // chassis
   Sprite::draw(b, (int)x, (int)y);
   next_frame();
   // stats
   draw_info(b, x - get_focus(get_frame()).x, y - get_focus(get_frame()).y);
   // controller display mouse cursors, etc.)
   controller->draw(b, (int)x, (int)y - 16);
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::update
////////////////////////////////////////////////////////////////////////////////
void Tank::update(void)
{
   fall();
   handle_input();

   fix y1 = (fix)highest_earth;
   fix y2 = (fix)highest_earth;
   fix y_change;
   while (getpixel(ground->get_data(), x - 16, y1) == 0) {
      y1++;
   }
   while (getpixel(ground->get_data(), x + 16, y2) == 0) {
      y2++;
   }
   y_change = y2 - y1;
   Object::angle = atan(y_change / 32);
   Sprite::angle = Object::angle;

   // check for collsion
   Singlist<Object>::Iterator current_object = objects->head;
   while (current_object != NULL) {
      if (collide(current_object->data->get_x(), current_object->data->get_y()) && (current_object->data->get_type() == TYPE_MISSILE)) {
         health--;
      }
      current_object = current_object->next;
   }

   // Check dead
   if (health <= 0) {
      exist = FALSE;
   }
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::check_steepness
////////////////////////////////////////////////////////////////////////////////
int Tank::check_steepness(int _x, int _y, int direction)
{
   int steepness;

   steepness = 0;
   while (getpixel(ground->get_data(), _x, _y) > 0) {
      _y--;
      steepness++;
   }

   return steepness;
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::move
////////////////////////////////////////////////////////////////////////////////
void Tank::move(fix change_x, fix change_y)
{
   int steepness;
   float _x, _y;

   _x = x + change_x;
   _y = y + change_y;
   steepness = check_steepness((int)_x, (int)_y - 1, SGN(change_x));
   if (steepness > 4) { // overly steep
      return;
   }
   if ((_x + 16 < ground->get_width()) && (_x - 16 >= 0)) {
      x = _x;
   }
   _y -= steepness;
   if ((_y < ground->get_height()) && (_y >= 0)) {
      y = _y;
   }
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::handleinput
////////////////////////////////////////////////////////////////////////////////
void Tank::handle_input(void)
{
   controller->poll();
   if (!weapon->get_loading()) {
      weapon->Object::set_angle(weapon->Object::get_angle() + controller->get_v());
      move(controller->get_h(), 0);
      if (weapon->Object::get_angle() > 255) {
         weapon->Object::set_angle(weapon->Object::get_angle() - 256);
      }
      if (weapon->Object::get_angle() < 0) {
         weapon->Object::set_angle(weapon->Object::get_angle() + 256);
      }
   }
   if (controller->get_a()) {
      weapon->load();
   }
   if ((weapon->get_loading() == TRUE) && !controller->get_a()) {
      weapon->shoot();
   }
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::fall
////////////////////////////////////////////////////////////////////////////////
void Tank::fall(void)
{
   int a;

   a = getpixel(ground->get_data(), x, y);
   if (y < ground->get_height()) {
      if (!(a > 0)) {
         y++;
      }
   }
}

////////////////////////////////////////////////////////////////////////////////
// Function: Tank::collide
////////////////////////////////////////////////////////////////////////////////
int Tank::collide(fix _x, fix _y)
{
   if ((_x > (x - (get_focus(get_frame()).x * scale))) &&
       (_x < (x - (get_focus(get_frame()).x * scale) + (get_w(get_frame()) * scale))) &&
       (_y > (y - (get_focus(get_frame()).y * scale))) &&
       (_y < (y - (get_focus(get_frame()).y * scale) + (get_h(get_frame()) * scale)))) {
      return 1;
   }
   return 0;
}

